searchData={"content_type":"text/markdown","items":[{"doc":"Helper functions for working with tailwind classes","ref":"TailColors.html","title":"TailColors","type":"module"},{"doc":"takes a list of classNames and a list of classes to remove, and then removes those classes if they appear in the classNames","ref":"TailColors.html#clean/2","title":"TailColors.clean/2","type":"function"},{"doc":"iex> TailColors.clean(\"thing needle something\", \"needle\")\n    \"thing something\"\n    iex> TailColors.clean(\"thing something\", \"needle\")\n    \"thing something\"\n    iex> TailColors.clean(\"thing needle haystack something\", \"needle haystack\")\n    \"thing something\"","ref":"TailColors.html#clean/2-examples","title":"Examples - TailColors.clean/2","type":"function"},{"doc":"takes a list of classNames and removes any colors that appear that do not follow the tailwind color struction prefix-color-tint\n#","ref":"TailColors.html#clean_colors/1","title":"TailColors.clean_colors/1","type":"function"},{"doc":"iex> TailColors.clean_colors(\"thing blue something\")\n    \"thing something\"\n    iex> TailColors.clean_colors(\"thing something\")\n    \"thing something\"\n    iex> TailColors.clean_colors(\"thing blue red something\")\n    \"thing something\"","ref":"TailColors.html#clean_colors/1-examples","title":"Examples - TailColors.clean_colors/1","type":"function"},{"doc":"takes a list of classNames and a list of classes to remove, and then removes those classes if they appear in the classNames","ref":"TailColors.html#clean_prefix/2","title":"TailColors.clean_prefix/2","type":"function"},{"doc":"iex> TailColors.clean_prefix(\"thing bg-blue-500 something\", \"bg\")\n    \"thing something\"\n    iex> TailColors.clean_prefix(\"thing something\", \"bg\")\n    \"thing something\"\n    iex> TailColors.clean_prefix(\"thing bg-green-200 text-red-600 something\", \"bg text\")\n    \"thing something\"","ref":"TailColors.html#clean_prefix/2-examples","title":"Examples - TailColors.clean_prefix/2","type":"function"},{"doc":"moves the tint down the list of tints.","ref":"TailColors.html#darker/2","title":"TailColors.darker/2","type":"function"},{"doc":"iex> TailColors.darker(600, 1)\n    700\n\n    iex> TailColors.darker(600, 3)\n    900\n\n    iex> TailColors.darker(600, 9)\n    950\n\n    iex> TailColors.darker(\"text-green-400\", 1)\n    \"text-green-500\"\n\n    iex> TailColors.darker(\"text-green-400\", 9)\n    \"text-green-950\"\n\n    iex> TailColors.darker(\"not-a-tint\", 1)\n    \"not-a-tint\"","ref":"TailColors.html#darker/2-examples","title":"Examples - TailColors.darker/2","type":"function"},{"doc":"takes list of classNames and prefix text or list of options, and finds first instance that matches,\na default value can also be set","ref":"TailColors.html#get/3","title":"TailColors.get/3","type":"function"},{"doc":"iex> TailColors.get(\"thing rounded something\", \"rounded\")\n    \"rounded\"\n\n    iex> TailColors.get(\"thing rounded-xl something\", \"rounded\")\n    \"rounded-xl\"\n\n    iex> TailColors.get(\"thing else\", \"rounded\")\n    nil\n\n    iex> TailColors.get(\"thing else\", \"rounded\", \"rounded-md\")\n    \"rounded-md\"\n\n    iex> TailColors.get(\"thing box else\", [\"circle\", \"rounded\", \"box\"])\n    \"box\"\n\n    iex> TailColors.get(\"thing else\", [\"circle\", \"rounded\", \"box\"])\n    nil\n\n    iex> TailColors.get(\"thing else\", [\"circle\", \"rounded\", \"box\"], \"rounded\")\n    \"rounded\"","ref":"TailColors.html#get/3-examples","title":"Examples - TailColors.get/3","type":"function"},{"doc":"takes a list of classNames and matches the first class with the prefix and returns the match or default values","ref":"TailColors.html#get/4","title":"TailColors.get/4","type":"function"},{"doc":"iex> TailColors.get(\"thing text-red-400 something\", \"text\", \"blue\", 600)\n    \"text-red-400\"\n\n    iex> TailColors.get(\"thing text-red something\", \"text\", \"blue\", 600)\n    \"text-red-600\"\n\n    iex> TailColors.get(\"thing something\", \"text\", \"blue\", 600)\n    \"text-blue-600\"","ref":"TailColors.html#get/4-examples","title":"Examples - TailColors.get/4","type":"function"},{"doc":"takes list of classNames and starting text, and finds first instance\nthat matches with a known color and tint, a default value can also be set","ref":"TailColors.html#get_color/3","title":"TailColors.get_color/3","type":"function"},{"doc":"iex> TailColors.get_color(\"thing text-red-400 something\", \"text\")\n    \"text-red-400\"\n\n    iex> TailColors.get_color(\"thing bg-blue\", \"bg\")\n    \"bg-blue\"\n\n    iex> TailColors.get_color(\"thing else\", \"bg\")\n    nil\n\n    iex> TailColors.get_color(\"thing bg-monster else\", \"bg\")\n    nil\n\n    iex> TailColors.get_color(\"thing bg-blue-404 else\", \"bg\")\n    nil\n\n    iex> TailColors.get_color(\"thing else\", \"bg\", \"bg-blue-600\")\n    \"bg-blue-600\"","ref":"TailColors.html#get_color/3-examples","title":"Examples - TailColors.get_color/3","type":"function"},{"doc":"takes a list of classNames and a string, and returns true if the string is in the list","ref":"TailColors.html#has?/2","title":"TailColors.has?/2","type":"function"},{"doc":"iex> TailColors.has?(\"thing text-red-400 something\", \"something\")\n    true\n\n    iex> TailColors.has?(\"thing bg-blue\", \"something\")\n    false","ref":"TailColors.html#has?/2-examples","title":"Examples - TailColors.has?/2","type":"function"},{"doc":"takes a either a tint, or a color class, and returns a tint that is easier to read on that background\n#","ref":"TailColors.html#invert/1","title":"TailColors.invert/1","type":"function"},{"doc":"iex> TailColors.invert(200)\n    600\n    iex> TailColors.invert(\"bg-blue-600\")\n    \"bg-blue-50\"\n    iex> TailColors.invert(nil)\n    nil\n    iex> TailColors.invert(\"weirdo\")\n    \"weirdo\"","ref":"TailColors.html#invert/1-examples","title":"Examples - TailColors.invert/1","type":"function"},{"doc":"moves the tint up the list of tints.","ref":"TailColors.html#lighter/2","title":"TailColors.lighter/2","type":"function"},{"doc":"iex> TailColors.lighter(600, 1)\n    500\n\n    iex> TailColors.lighter(600, 3)\n    300\n\n    iex> TailColors.lighter(600, 9)\n    50\n\n    iex> TailColors.lighter(\"text-green-400\", 1)\n    \"text-green-300\"\n\n    iex> TailColors.lighter(\"text-green-400\", 9)\n    \"text-green-50\"\n\n    iex> TailColors.lighter(\"not-a-tint\", 1)\n    \"not-a-tint\"","ref":"TailColors.html#lighter/2-examples","title":"Examples - TailColors.lighter/2","type":"function"},{"doc":"takes a list of classNames and finds any colors that appear that are not prefixed by a string followed by a -\nreturns a tuple of {color, tint}","ref":"TailColors.html#main_color/3","title":"TailColors.main_color/3","type":"function"},{"doc":"iex> TailColors.main_color(\"thing red something\", \"blue\", 700)\n    {\"red\", 700}\n\n    iex> TailColors.main_color(\"thing red-400 something\", \"blue\", 700)\n    {\"red\", 400}\n\n    iex> TailColors.main_color(\"thing something\", \"blue\", 700)\n    {\"blue\", 700}","ref":"TailColors.html#main_color/3-examples","title":"Examples - TailColors.main_color/3","type":"function"},{"doc":"substitutes a themed color from config for the actual color name.","ref":"TailColors.html#theme/1","title":"TailColors.theme/1","type":"function"},{"doc":"# TailColors\n\nThis library is a collection of useful functions for working with tailwind colors.  It is meant to be a helper library if you're building out your own custom components that utilizes class names to key logic off of.  The paradigm is heavily influenced by [DaisyUI](https://daisyui.com/components/button/), however, you must create the components.\n\nThe main advantage of this approach is that you can create complex components with nested elements, and then easily pass in style overrides for those NESTED elements in the class string, instead of having to build out dozens of props. or tons of slots.  Also, it keeps all styling information in the class attribute.  This makes it easier to see what styles are being applied. Props then can be used for data, and slots can be used for content.\n```elixir\n Hello \n```\nvs\n```elixir\n \n   \n     \n      Hello\n     \n   \n     \n        \n     \n  Hello \n```\nvs\n```elixir\n Hello \n```","ref":"readme.html","title":"TailColors","type":"extras"},{"doc":"```elixir\n@deps [\ntail_colors: \"~> 0.1.0\"\n]\n```","ref":"readme.html#installation","title":"Installation - TailColors","type":"extras"},{"doc":"First configure tail_color schemes in config.exs\n```elixir\nimport Config\n\nconfig :tail_colors,\n  primary: \"purple\",\n  secondary: \"blue\",\n  accent: \"yellow\",\n  info: \"sky\",\n  success: \"green\",\n  warning: \"orange\",\n  error: \"red\",\n```\nIf you are using custom colors in your tailwind.config.js file, you can add those colors so they will identified as colors to identify in your class names:\n```elixir\nimport Config\n\nconfig :tail_colors,\n  colors: [\n    \"hawthorne\",\n    \"midnight\"\n  ],\n```","ref":"readme.html#config","title":"Config - TailColors","type":"extras"},{"doc":"TailColors provides a few functions to help you build out your own components where you pass in data through the class attribute.  For example, if you want to build a button component, you can pass in the color, style, and size of the button through the class attribute, and then use TailColors to parse out the data, and modify the classes.\n\n##","ref":"readme.html#usage","title":"Usage - TailColors","type":"extras"},{"doc":"```\n Hello \n```\n\nNotice how we are passing in the \"primary\" class.  TailColors will automatically fetch primary color from the config file.\n\n##","ref":"readme.html#home_live-html-leex","title":"home_live.html.leex - TailColors","type":"extras"},{"doc":"```\ndefmodule MyApp.Button do\n  use Phoenix.Component\n  import TailColors\n  @sizes ~w\"xs sm md lg xl\"\n  @styles ~w\"classic outline\"\n  @default_color Map.get(Application.compile_env(:tail_colors, :themed_colors), :primary)\n  @default_tint 700\n\n  attr :class, :string, default: \"\"\n  attr :rest, :gobal\n  slot :inner_block, required: true\n  def button(assigns) do\n    ~H\"\"\"\n     \n       \n     \n    \"\"\"\n  end\n\n  defp parse_class(class_str) do\n    class_str = theme(class_str)\n    l = String.split(class_str, ~r/\\s+/)\n\n    size = get(l, @sizes, \"md\")\n    style = get(l, @styles, \"classic\")\n    {color, tint} = main_color(l, @default_color, @default_tint)\n\n    [case size do\n      \"xs\" -> \"px-2.5 py-1.5 text-xs\"\n      \"sm\" -> \"px-3 py-2 text-sm\"\n      \"md\" -> \"px-4 py-2 text-md\"\n      \"lg\" -> \"px-4 py-2 text-lg\"\n      \"xl\" -> \"px-6 py-3 text-xl\"\n    end]\n    ++\n    case style do\n      \"classic\" ->\n        [\n          get(l, \"bg\", color, tint),\n          \"hover:#{get(l, \"bg\", color, tint) |> darker(2)}\",\n          get(l, \"ring\", color, tint),\n          get(l, \"text\", color, invert(tint))\n        ]\n        \"outline\" ->\n          [\n            \"hover:#{get(l, \"bg\", color, tint)} hover:#{get(l, \"text\", color, tint) |> invert()}\",\n            get(l, \"ring\", color, tint),\n            \"border #{get(l, \"border\", color, tint)}\",\n            get(l, \"text\", color, tint)\n          ]\n    end\n    ++ [\n          clean(class_str, @sizes ++ @styles)\n          |> clean_prefix(~w\"bg text, ring\")\n          |> clean_colors()\n    ]\n  end\nend\n```\n\nOK, let's walk through that together.  At the top of the module, we import TailColors, and set the accepted options for size and styles.  We also set the default_color to pull from the primary attribute from the config.  Finally, we set the default_tint.\n\nNext, we have a standard button component, however, in the class attribute, we're calling the parse_class function, and passing in the class string.\n\nThe parse_class function is where all of our major logic lies.  The first thing that we do is we replace the class_str with the theme(class_str).  This function simply looks through all the classes and replaces any themed_colors with their corresponding values.  So in the example above, since we set primary to purple in the config, the class_str would be replaced with \"purple box xl\". `Please note`: this performs string replace on every theme_color, performance could become an issue, use with caution.\n\nNext, we create a list out of the class_str.  Technically speaking, each of the functions in TailColors can accept either a string or a list, however, since all the functions just convert the string into a list anyways, we'll save ourselves some cpu's and just pass in the list.\n\nThis is where things start to get interesting.  We use the `TailColors.get/3` function to find out if any of the known sizes defined in *@sizes is present in the class list.  If it is, we set the size variable to that value, otherwise, we set it to the default value that's passed in the third parameter, in this case: \"md\".  We do the same thing for style, except we look for the values in @styles.\n\nThe `TailColors.main_color/3` function looks through the class list for any defined colors, either default colors that come with tailwind, or custom colors that you've defined in your tailwind.config.js file.  If it finds a color, it returns the color, and the tint in a tuple.  If it doesn't find a color, it returns the default_color and default_tint which are passed in as the second and third arguments.\n\nin the example above, the only color that was passed in was 'primary', which was replaced by \"purple\" by the theme function, and there's no tint so the default tint is used, which in this case is `700`.  If a tint was passed in, such as `primary-200`, then the `200` would have been returned as the tint.\n\nNext, we start building out the list of strings that we are going to return.  Phoenix will convert this list into a string, so it will be an accepted class attribute.  The `case` statement around size is pretty self-explanatory.  However, the case for style is way more interesting.\n\n`TailColors.get/4` function takes the class list, a prefix to use as an identifier and a default value to use for the color and the tint.  The function will then search through the class list looking for any classes that start with the prefix, if it finds the prefix, it returns the full class name: `prefix-color-tint`.  If it doesn't find the prefix, it returns the default value with the prefix.  Here are a few examples:\n```elixir\nget(~w\"one bg-red-100 two\", \"bg\", \"blue\", 500)\n\"bg-red-100\"\n\nget(~w\"one bg-red two\", \"bg\", \"blue\", 500)\n\"bg-red-500\"\n\nget(~w\"one two\", \"bg\", \"blue\", 500)\n\"bg-blue-500\"\n```\n\nThis means that we can **OVERIDE** the default color and tint by passing in a class name that starts with the prefix. This works for any standard tailwind classname that follows the `prefix-color-tint` model.  In the example button component above, the `get/4` function is looking to see if there is a classname that matches `text-color-tint`, and if there is, it uses that value, otherwise, it uses the default color and tint.\n\nNext, there's a weird `\"hover:#{get(l, \"bg\", color, tint) |> darker(2)}\"` line of code.  This is using string interpolation to build out the hover class.  The `TailColors.get/4` function we've already talked about.  But the `TailColors.darker/2` function is new.  It takes a standard tailwind class in the `prefix-color-tint` format, and the number of steps on the tailwind color tint array to use.  Here's some examples:\n```elixir\ndarker(\"text-red-600\", 2)\n\"text-red-800\"\n\ndarker(\"ring-red-600\", 8)\n\"ring-red-950\"\n```\nNotice when we try to step past the darkest shade, it just returns the darkest shade available.  As you can probably guess, there's also a `TailColors.lighter/2` function that works the same way.\n\nFinally, there's the `TailColors.invert/1` function.  This function takes a tailwind prefix-color-tint class, and returns the inverted color tint class.  For example:\n```elixir\ninvert(\"text-red-600\")\n\"text-red-50\"\n\ninvert(\"text-red-100\")\n\"text-red-500\"\n```\nThe `invert/1` function is primarily used to invert the text color for better readability.\n\nAt the bottom of the `parse/1` function, there are several `clean` functions.  These functions remove classnames from the class string to avoid polluting the class string.  For instance, once we've identified the \"outline\" style, we don't want \"outline\" to appear in our class string, on the off chance that somewhere else, there is an \"outline\" css class.  However, we may want that, so you can choose to use the clean functions or not.\n\n`TailColors.clean/2` takes the class string, and a list of classnames to remove from the class string.  `TailColors.clean_prefix/2` takes the class string, and a list of prefixes to remove from the class string.  `TailColors.clean_colors/1` takes the class string, and removes any stand alone color classes from the class string.\n\nI would recommend using the `TailColors.clean_prefix/2` function if you are using any interpolated classes, such as \"hover:bg-red-600\" where the color and tint are provided in the \"bg-red-600\" in the class name.  Otherwise, you'll end up with \"hover:bg-red-600 bg-red-600\" in your class string, which will cause the background to always be red! instead of just on hover.","ref":"readme.html#lib-my_app_web-components-button-ex","title":"lib/my_app_web/components/button.ex - TailColors","type":"extras"},{"doc":"Tailwind looks for known classes to appear in your application's source code.  However, our source code is using dynamically generated class names.  So tailwind doesn't know to include it in the final css file.  To fix this, we need to add the following to our tailwind.config.js file:\n```js\nsafelist: [\n  \"text-red-600\",\n  \"ring-blue-400\",\n  \"hover:ring-blue-600\",\n  ...\n]\n```\n**@@TODO**:\n- [ ] Create a mix task that will automatically generate a list of all the classes that are used in your application for the safelist setting.\n- [ ] Create a mix task that will take a directory and do a find and replace inside any class attributes so you can replace a color setting across your entire application.","ref":"readme.html#help-i-m-not-seeing-my-changes-take-effect","title":"HELP! I'm not seeing my changes take effect! - TailColors","type":"extras"},{"doc":"MIT License\n\n----\nCreated:  2023-11-06Z","ref":"readme.html#license","title":"License - TailColors","type":"extras"}]}